import random


cdef class RandomNormalizedArray(object):

    cdef list __array

    def __init__(self, itemCount: int):
        """
        The constructor of RandomNormalizedArray class gets an integer itemCount as an input. Creates a list of
        size itemCount and loops through each element of the list and initializes them with a random number by using
        random. Then, accumulates each element of the list and at the end divides each element with the resulting sum.

        Parameters
        ----------
        itemCount : int
            input defining array size.
        """
        cdef double total
        cdef int i
        total = 0.0
        self.__array = []
        for i in range(itemCount):
            self.__array.append(random.uniform(0, 1))
            total += self.__array[i]
        for i in range(itemCount):
            self.__array[i] /= total

    cpdef list get(self):
        """
        Getter for the double list.

        Returns
        ----------
        list
            the double list.
        """
        return self.__array



cdef class Swap(object):

    @staticmethod
    def swap(array: list, i: int, j: int):
        """
        The swap method takes a list and two integer numbers i, j. And interchange the given array's
        items at index i and index j.

        Parameters
        ----------
        array : list
            input
        i : int
            first index to swap.
        j : int
            second index to change.
        """
        t = array[i]
        array[i] = array[j]
        array[j] = t



from Util.Swap import Swap


cdef class Permutation(object):

    cdef list __a
    cdef int __n
    
    def __init__(self, n: int):
        """
        A constructor of Permutation class which creates a new list and assigns integer
        numbers starting from 0 to given input n.

        Parameters
        ----------
         n: int
            integer input.
         """
        cdef int i
        self.__n = n
        self.__a = []
        for i in range(n):
            self.__a.append(i)

    cpdef list get(self):
        """
        The get method returns the list a.

        Returns
        ----------
        list
            a
        """
        return self.__a

    cpdef bint next(self):
        """
         The next method generates next permutation for the list a.

        Returns
        ----------
        boolean
            true if next permutation is possible, false otherwise.
        """
        cdef int i, j, k
        i = self.__n - 2
        while i >= 0 and self.__a[i] >= self.__a[i + 1]:
            i = i - 1
        if i == -1:
            return False
        j = self.__n - 1
        while self.__a[i] >= self.__a[j]:
            j = j - 1
        Swap.swap(self.__a, i, j)
        k = i + 1
        j = self.__n - 1
        while k < j:
            Swap.swap(self.__a, j, k)
            k = k + 1
            j = j - 1
        return True



from Util.Subset cimport Subset

cdef class SubsetFromList(Subset):

    cdef list __elementList
    cdef list __indexList

    def __init__(self, _list: list, elementCount: int):
        """
        A constructor of SubsetFromList class takes an integer list and an integer elementCount as inputs. It
        initializes elementList and elementCount variables with given inputs, then creates 3 arrays; set,indexList, and
        multiset with the size of given elementCount and multisetCount, which is derived from elementCount,
        respectively. Then, it assigns i to each ith element of indexList list and use this index to point at
        elementList and assigns to set list.

        Parameters
        ----------
        _list : list
            list type input.
        elementCount : int
            input element count.
        """
        cdef int i
        self.__elementList = _list
        self.elementCount = elementCount
        self.set = []
        self.__indexList = []
        for i in range(elementCount):
            self.__indexList.append(i)
            self.set.append(self.__elementList[self.__indexList[i]])

    cpdef bint next(self):
        """
        The next method generates the next subset from list.

        Returns
        ----------
        boolean
            true if next subset generation from list is possible, false otherwise.
        """
        cdef int i, j
        for i in range(self.elementCount - 1, -1, -1):
            self.__indexList[i] = self.__indexList[i] + 1
            if self.__indexList[i] < len(self.__elementList) - self.elementCount + i + 1:
                break
        else:
            return False
        self.set[i] = self.__elementList[self.__indexList[i]]
        for j in range(i + 1, self.elementCount):
            self.__indexList[j] = self.__indexList[j - 1] + 1
            self.set[j] = self.__elementList[self.__indexList[j]]
        return True



from distutils.core import setup
from Cython.Build import cythonize

setup(
    ext_modules=cythonize(["Util/*.pyx", "Util/*.pxd"], compiler_directives={'language_level' : "3"}),
    name='NlpToolkit-Util-Cy',
    version='1.0.3',
    packages=['Util'],
    package_data={'Util': ['*.pxd', '*.pyx', '*.c']},
    url='https://github.com/olcaytaner/Util-Cy',
    license='',
    author='olcaytaner',
    author_email='olcaytaner@isikun.edu.tr',
    description='Simple Utils'
)
